"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const webpack = require("webpack");
const fs = require("fs");
const path = require("path");
const chalk = require("chalk");
const child_process_1 = require("child_process");
const webpack_config_1 = require("../config/webpack-config");
process.env.NODE_ENV = 'development';
let childProcess = null;
let chunkPath = '';
let buildTime = 0;
let hasError = false;
const logger = (...args) => {
    console.log(chalk.blueBright(...args));
};
function createChildProcess() {
    childProcess = child_process_1.fork(chunkPath, [], { stdio: 'inherit' });
    childProcess.on('error', (err) => {
        console.error(err.message);
        process.exit(1);
    });
    childProcess.on('close', () => {
        var _a;
        console.clear();
        if (((_a = childProcess) === null || _a === void 0 ? void 0 : _a.killed) && !hasError) {
            createChildProcess();
        }
    });
    logger(`compile succeed! Done in ${(buildTime / 1000).toFixed(2)}s`);
}
function build(configs) {
    const compiler = webpack(configs);
    compiler.hooks.watchRun.tap('ssr-watch-run', () => {
        console.clear();
        logger('compiling...');
    });
    compiler.watch({ ignored: /node_module/ }, (err, stats) => {
        var _a, _b, _c;
        if (err) {
            hasError = true;
            (_a = childProcess) === null || _a === void 0 ? void 0 : _a.kill(1);
            throw err;
        }
        const [serverStats] = stats.stats;
        const { errors, warnings, outputPath, assetsByChunkName, time } = serverStats.toJson();
        if (warnings.length) {
            console.log(chalk.yellowBright('compiled with warning:'));
            warnings.forEach(console.warn);
        }
        if (errors.length) {
            console.log(chalk.redBright('compiled with error:'));
            errors.forEach(console.error);
            hasError = true;
            (_b = childProcess) === null || _b === void 0 ? void 0 : _b.kill(1);
            process.exit(1);
        }
        hasError = false;
        const assets = (_c = assetsByChunkName) === null || _c === void 0 ? void 0 : _c.app;
        if (!assets || assets[0].includes('client')) {
            // 说明当前打包的是客户端代码
            return;
        }
        chunkPath = path.join(((outputPath !== null && outputPath !== void 0 ? outputPath : '')).toString(), assets[0]);
        buildTime = time;
        if (childProcess) {
            childProcess.kill();
        }
        else {
            createChildProcess();
        }
    });
}
function rmdir(directory) {
    if (directory && fs.existsSync(directory)) {
        const files = fs.readdirSync(directory);
        if (files.length) {
            files.forEach(filename => {
                const filePath = path.join(directory, filename);
                const stats = fs.statSync(filePath);
                if (stats.isDirectory()) {
                    rmdir(filePath);
                }
                else {
                    fs.unlinkSync(filePath);
                }
            });
        }
        fs.rmdirSync(directory);
    }
}
function buildEveryThing() {
    var _a;
    const serverDevConfig = webpack_config_1.createWebpackConfig(true);
    const clientDevConfig = webpack_config_1.createWebpackConfig(false);
    rmdir((_a = serverDevConfig.output) === null || _a === void 0 ? void 0 : _a.path);
    build([serverDevConfig, clientDevConfig]);
}
buildEveryThing();
