import * as React from 'react'
import * as H from 'history'
import * as types from 'src/types'
import { parse as parseUrl, Url } from 'url'
import { pathToRegexp } from 'path-to-regexp'
import { findMatchedRoute, callGetInitialProps } from 'src/utils'

const RouterContext = React.createContext({} as H.Location)

export const history = 'undefined' === typeof window
  ? H.createMemoryHistory()
  : H.createBrowserHistory()

export const useLocation = () => {
  return React.useContext(RouterContext)
}

export const Router: React.FC<types.RouterProps> = props => {
  const component = React.useRef<React.ComponentType<any> | null>(props.component)
  const pageProps = React.useRef(props.pageProps)
  const [location, setLocation] = React.useState<H.Location>(() => {
    const parsedUrl: Url = parseUrl(props.url)
    return {
      pathname: parsedUrl.pathname || '/',
      search: parsedUrl.search || '',
      hash: parsedUrl.hash || '',
      state: null,
    }
  })

  React.useEffect(
    () => {
      const unlisten = history.listen(async (newLocation: H.Location) => {
        console.log(newLocation.pathname, location.pathname)
        if (newLocation.pathname === location.pathname) {
          return
        }
        const matched = findMatchedRoute(
          newLocation.pathname, 
          props.routes
        )
        if (matched) {
          const initialProps = await callGetInitialProps(
            props.App,
            matched.component,
            newLocation.pathname,
          )
          const params = pathToRegexp(matched.path).exec(newLocation.pathname)
          console.log('params is', params)
          pageProps.current = initialProps
          component.current = matched.component
        } else {
          component.current = null
        }
        setLocation(newLocation)
      })
      return unlisten
    },
    [location.pathname]
  )

  // const App = props.App

  return (
    <RouterContext.Provider value={location}>
      <props.App 
        {...pageProps.current}
        Component={component.current}
      />
    </RouterContext.Provider>
  )
}
