import * as React from 'react'
import * as fs from 'fs'
import * as cheerio from 'cheerio'
import * as types from 'src/types'
import { renderToString as originalRenderToString } from 'react-dom/server'
import { getConfig } from 'config/config'
import { findMatchedRoute, callGetInitialProps } from 'src/utils'
import Root from 'src/Root'
import DefaultApp from 'src/App'

const config = getConfig()
const template = fs.readFileSync(config.htmlTemplatePath, 'utf8')

function writeDataToHTML($: CheerioStatic, pageProps: object) {
  const data = JSON.stringify({
    pageProps,
  })
  $('head').append(
    `<script type="text/javascript">
        window.__APP_DATA__ = ${data}
    </script>`
  )
}

function appendScriptOnDevelop($: CheerioStatic) {
  if (config.isDev) {
    $('body').append(`
      <script type="text/javascript" src="/static/app.js"></script>
    `)
  }
}

export async function renderToString(url: string, options: types.RenderOptions): Promise<string> {
  const $ = cheerio.load(template)
  const matched = findMatchedRoute(url, options.routes)
  const App = options.App || DefaultApp
  const initialProps = await callGetInitialProps(
    App,
    matched && matched.component,
    url,
  )
  const content = originalRenderToString(
    <Root
      url={url}
      routes={options.routes || []}
      component={matched ? matched.component : null}
      pageProps={initialProps}
      App={App}
    />
  )
  $(options.container).html(content)
  writeDataToHTML($, initialProps)
  appendScriptOnDevelop($)
  return $.html()
}
